==> srtp-1.0.1.tgz: srtp-1.0.1/srtp/rtp.c <==
/*
 * rtp.c
 *
 * library functions for the real-time transport protocol
 *
 * David A. McGrew
 * Cisco Systems, Inc.
 */


#include "rtp.h"

#include <stdio.h>
#include <string.h>

#include <sys/types.h>
#include <sys/socket.h>

#define PRINT_DEBUG    0    /* set to 1 to print out debugging data */
#define VERBOSE_DEBUG  0    /* set to 1 to print out more data      */

ssize_t
rtp_sendto(rtp_sender_t *sender, const void* msg, int len) {
  int octets_sent;
  err_status_t stat;
  int pkt_len = len + rtp_header_len;

  /* marshal data */
  strncpy(sender->message.body, msg, len);
  
  /* update header */
  sender->message.header.seq++;
  sender->message.header.ts++;
  
  /* apply srtp */
  stat = srtp_protect(&sender->srtp_ctx, &sender->message.header, &pkt_len);
  if (stat) {
#if PRINT_DEBUG
    fprintf(stderr, "error: srtp protection failed with code %d\n", stat);
#endif
    return -1;
  }
#if VERBOSE_DEBUG
  srtp_print_packet(&sender->message.header, pkt_len);
#endif
  octets_sent = sendto(sender->socket, &sender->message,
		       pkt_len, 0, (struct sockaddr *)&sender->addr,
		       sizeof (struct sockaddr_in));

  if (octets_sent != pkt_len) {
#if PRINT_DEBUG
    fprintf(stderr, "error: couldn't send message %s", (char *)msg);
    perror("");
#endif
  }

  return octets_sent;
}

ssize_t
rtp_recvfrom(rtp_receiver_t *receiver, void *msg, int *len) {
  int octets_recvd;
  err_status_t stat;
  
  octets_recvd = recvfrom(receiver->socket, (void *)&receiver->message,
			 *len, 0, (struct sockaddr *) NULL, 0);

  /* verify rtp header */
  if (receiver->message.header.version != 2) {
    *len = 0;
    return -1;
  }

#if PRINT_DEBUG
  fprintf(stderr, "%d octets received from SSRC %d",
	  octets_recvd, receiver->message.header.ssrc);
#endif
#if VERBOSE_DEBUG
  srtp_print_packet(&receiver->message.header, octets_recvd);
#endif

  /* apply srtp */
  stat = srtp_unprotect(&receiver->srtp_ctx,
			&receiver->message.header, &octets_recvd);
  if (stat) {
#if PRINT_DEBUG
    fprintf(stderr,
	    "\nerror: srtp unprotection failed with code %d", stat);
    if (stat == err_status_replay_fail)
      fprintf(stderr, " (replay check failed)\n");
    if (stat == err_status_auth_fail)
      fprintf(stderr, " (auth check failed)\n"); 
#endif
    return -1;
  }
  strncpy(msg, receiver->message.body, octets_recvd);
  
  return octets_recvd;
}

int
rtp_sender_init(rtp_sender_t *sender, int socket, struct sockaddr_in addr) {

  /* set header values */
  sender->message.header.ssrc    = random();
  sender->message.header.ts      = 0;
  sender->message.header.seq     = (uint16_t) random();
  sender->message.header.m       = 0;
  sender->message.header.pt      = 0x8f;
  sender->message.header.version = 2;
  sender->message.header.p       = 0;
  sender->message.header.x       = 0;
  sender->message.header.cc      = 0;

  /* set other stuff */
  sender->socket = socket;
  sender->addr = addr;

  return 0;
}

int
rtp_receiver_init(rtp_receiver_t *rcvr, int socket, struct sockaddr_in addr) {
  
  /* set header values */
  rcvr->message.header.ssrc    = 0;
  rcvr->message.header.ts      = 0;
  rcvr->message.header.seq     = 0;
  rcvr->message.header.m       = 0;
  rcvr->message.header.pt      = 0x8f;
  rcvr->message.header.version = 2;
  rcvr->message.header.p       = 0;
  rcvr->message.header.x       = 0;
  rcvr->message.header.cc      = 0;

  /* set other stuff */
  rcvr->socket = socket;
  rcvr->addr = addr;

  return 0;
}


/*
 * the functions below provide an interface between rtp and srtp
 *
 */


/*
 * declare external crypto objects for use below
 */

extern cipher_type_t null_cipher;
extern cipher_type_t rijndael_icm;
extern auth_type_t   null_auth;
extern auth_type_t   tmmhv2;


/*
 * srtp_init_from_hex(...) is an internal function used in
 * srtp_sender_init(...) and srtp_receiver_init(...)
 */

#define SRTP_INIT_MAX_KEY_LEN 64

int
srtp_init_from_hex(srtp_ctx_t *ctx, sec_serv_t sec_servs,
		   unsigned char *input_key, uint32_t ssrc) {
  unsigned char key[SRTP_INIT_MAX_KEY_LEN];
  unsigned char *salt;
  err_status_t stat;
  int len, len_expected;
  
  /* 
   * use manual keying - read key and salt from hexadecimal input
   * string
   */

  len_expected = 30 * 2;  /* 16 + 14 octets of key + salt expected */

  /* read hex string to key/salt buffer */
  len = hex_string_to_octet_string(key, input_key, len_expected);

  /* check that hex string is the right length */
  if (len < len_expected) {
    fprintf(stderr, 
	    "error: too few digits in key/salt "
	    "(should be %d hexadecimal digits, found %d)\n",
	    len_expected, len);
    exit(1);    
  } 
  if (strlen(input_key) > len_expected) {
    fprintf(stderr, 
	    "error: too many digits in key/salt "
	    "(should be %d hexadecimal digits, found %ld)\n",
	    len_expected, strlen(input_key));
    exit(1);    
  }
    
  /* set the salt pointer to the correct location in the key/salt string */
  salt = key + 16;

  printf("set master key/salt to %s/", octet_string_hex_string(key, 32));
  printf("%s\n", octet_string_hex_string(salt, 28));
   
  stat = srtp_init_aes_128_prf(ctx, key, salt, ssrc);
  if (stat)
    fprintf(stderr, "error: srtp initialization failed with code %d\n", stat);

  return 0;
}

int
srtp_sender_init(rtp_sender_t *sender, unsigned int ssrc, 
		 struct sockaddr_in name, sec_serv_t sec_servs,
		 unsigned char *input_key, unsigned int group_id) {
  cipher_type_t *ctype;
  auth_type_t *atype;
  int cipher_key_len = 0;
  int auth_key_len = 0;
  int auth_tag_len = 0;
  err_status_t stat;  
#if (SRTP_GDOI == 1)
  gdoi_policy_t policy;
  gdoi_keys_t keys;

  memset((void *)&policy, 0, sizeof(gdoi_policy_t));
  memset((void *)&keys, 0, sizeof(gdoi_keys_t));
#endif

  /*
   * cipher and auth func are hardwired in this function for
   * simplicity's sake
   */
  
  /* choose cipher based on security service flag */
  if (sec_servs) {
    ctype = &rijndael_icm;
    cipher_key_len = 16;
  } else {
    ctype = &null_cipher;
  }
  
  /* choose auth_func based on security service flag */
  if (sec_servs & sec_serv_auth) {
    atype = &tmmhv2;
    auth_key_len = 94;
    auth_tag_len = 4;
  } else {
    atype = &null_auth;
  }
  
  /* allocate srtp context */
  stat = srtp_alloc(&sender->srtp_ctx,
		    ctype,
		    cipher_key_len,
		    atype,
		    auth_key_len,
		    auth_tag_len,
		    sec_servs);
  if (stat)
    fprintf(stderr, "error: srtp alloc failed with code %d\n", stat);

  if (group_id) {
#if (SRTP_GDOI == 1)
    srtp_gdoi_get_policy_and_keys(group_id, &name, ssrc, &policy, &keys);
    /* Set the ssrc to the one specified on the command line. */
    sender->message.header.ssrc    = ssrc;
#else
    fprintf(stderr, "error: gdoi not available\n");

#endif
  }

  if (sec_servs) {
#if (SRTP_GDOI == 1)
    srtp_gdoi_crypto_init(&sender->srtp_ctx, &sender->message, sec_servs
			  input_key, &policy, &keys);
#else
    srtp_init_from_hex(&sender->srtp_ctx, sec_servs, input_key, ssrc);
#endif
  }   
    
  return 0;

}


int
srtp_receiver_init(rtp_receiver_t *rcvr, unsigned int ssrc,
		   struct sockaddr_in name, sec_serv_t sec_servs,
		   unsigned char *input_key, unsigned int group_id) {
  err_status_t stat;
  cipher_type_t *ctype = &null_cipher;
  auth_type_t *atype;
  int cipher_key_len = 0;
  int auth_key_len = 0;
  int auth_tag_len = 0;
#if (SRTP_GDOI)
  gdoi_policy_t policy;
  gdoi_keys_t keys;

  memset((void *)&policy, 0, sizeof(gdoi_policy_t));
  memset((void *)&keys, 0, sizeof(gdoi_keys_t));
#endif

  /* choose cipher based on security service flag */
  if (sec_servs) {
    ctype = &rijndael_icm;
    cipher_key_len = 16;
  } else {
    ctype = &null_cipher;
  }
  
  /* choose auth_func based on security service flag */
  if (sec_servs & sec_serv_auth) {
    atype = &tmmhv2;
    auth_key_len = 94;
    auth_tag_len = 4;
  } else {
    atype = &null_auth;
  }
  
  /* allocate srtp context */
  stat = srtp_alloc(&rcvr->srtp_ctx,
		    ctype,
		    cipher_key_len,
		    atype,
		    auth_key_len,
		    auth_tag_len,
		    sec_servs);
  if (stat)
    fprintf(stderr, "error: srtp alloc failed with code %d\n", stat);

#if (SRTP_GDOI == 1)
  if (group_id)
    srtp_gdoi_get_policy_and_keys(group_id, &name, ssrc, &policy, &keys);
#endif


  if (sec_servs) {
#if (SRTP_GDOI == 1)
    srtp_crypto_init(&rcvr->srtp_ctx, &rcvr->message,
    		     sec_servs, input_key, &policy, &keys);
#else
    srtp_init_from_hex(&rcvr->srtp_ctx, sec_servs, input_key, ssrc);
#endif
  }
  
  return 0;
}

==> srtp-1.0.5.tgz: srtp-1.0.5/srtp/rtp.c <==
/*
 * rtp.c
 *
 * library functions for the real-time transport protocol
 *
 * David A. McGrew
 * Cisco Systems, Inc.
 */


#include "rtp.h"

#include <stdio.h>
#include <string.h>

#include <sys/types.h>
#include <sys/socket.h>

#define PRINT_DEBUG    0    /* set to 1 to print out debugging data */
#define VERBOSE_DEBUG  0    /* set to 1 to print out more data      */

ssize_t
rtp_sendto(rtp_sender_t *sender, const void* msg, int len) {
  int octets_sent;
  err_status_t stat;
  int pkt_len = len + rtp_header_len;

  /* marshal data */
  strncpy(sender->message.body, msg, len);
  
  /* update header */
  sender->message.header.seq++;
  sender->message.header.ts++;
  
  /* apply srtp */
  stat = srtp_protect(sender->srtp_ctx, &sender->message.header, &pkt_len);
  if (stat) {
#if PRINT_DEBUG
    fprintf(stderr, "error: srtp protection failed with code %d\n", stat);
#endif
    return -1;
  }
#if VERBOSE_DEBUG
  srtp_print_packet(&sender->message.header, pkt_len);
#endif
  octets_sent = sendto(sender->socket, &sender->message,
		       pkt_len, 0, (struct sockaddr *)&sender->addr,
		       sizeof (struct sockaddr_in));

  if (octets_sent != pkt_len) {
#if PRINT_DEBUG
    fprintf(stderr, "error: couldn't send message %s", (char *)msg);
    perror("");
#endif
  }

  return octets_sent;
}

ssize_t
rtp_recvfrom(rtp_receiver_t *receiver, void *msg, int *len) {
  int octets_recvd;
  err_status_t stat;
  
  octets_recvd = recvfrom(receiver->socket, (void *)&receiver->message,
			 *len, 0, (struct sockaddr *) NULL, 0);

  /* verify rtp header */
  if (receiver->message.header.version != 2) {
    *len = 0;
    return -1;
  }

#if PRINT_DEBUG
  fprintf(stderr, "%d octets received from SSRC %d",
	  octets_recvd, receiver->message.header.ssrc);
#endif
#if VERBOSE_DEBUG
  srtp_print_packet(&receiver->message.header, octets_recvd);
#endif

  /* apply srtp */
  stat = srtp_unprotect(receiver->srtp_ctx,
			&receiver->message.header, &octets_recvd);
  if (stat) {
#if PRINT_DEBUG
    fprintf(stderr,
	    "\nerror: srtp unprotection failed with code %d", stat);
    if (stat == err_status_replay_fail)
      fprintf(stderr, " (replay check failed)\n");
    if (stat == err_status_auth_fail)
      fprintf(stderr, " (auth check failed)\n"); 
#endif
    return -1;
  }
  strncpy(msg, receiver->message.body, octets_recvd);
  
  return octets_recvd;
}

int
rtp_sender_init(rtp_sender_t *sender, int socket, struct sockaddr_in addr) {

  /* set header values */
  sender->message.header.ssrc    = random();
  sender->message.header.ts      = 0;
  sender->message.header.seq     = (uint16_t) random();
  sender->message.header.m       = 0;
  sender->message.header.pt      = 0x8f;
  sender->message.header.version = 2;
  sender->message.header.p       = 0;
  sender->message.header.x       = 0;
  sender->message.header.cc      = 0;

  /* set other stuff */
  sender->socket = socket;
  sender->addr = addr;

  return 0;
}

int
rtp_receiver_init(rtp_receiver_t *rcvr, int socket, struct sockaddr_in addr) {
  
  /* set header values */
  rcvr->message.header.ssrc    = 0;
  rcvr->message.header.ts      = 0;
  rcvr->message.header.seq     = 0;
  rcvr->message.header.m       = 0;
  rcvr->message.header.pt      = 0x8f;
  rcvr->message.header.version = 2;
  rcvr->message.header.p       = 0;
  rcvr->message.header.x       = 0;
  rcvr->message.header.cc      = 0;

  /* set other stuff */
  rcvr->socket = socket;
  rcvr->addr = addr;

  return 0;
}


/*
 * the functions below provide an interface between rtp and srtp
 *
 */


/*
 * declare external crypto objects for use below
 */

extern cipher_type_t null_cipher;
extern cipher_type_t rijndael_icm;
extern auth_type_t   null_auth;
extern auth_type_t   tmmhv2;


/*
 * srtp_init_from_hex(...) is an internal function used in
 * srtp_sender_init(...) and srtp_receiver_init(...)
 */

#define SRTP_INIT_MAX_KEY_LEN 64

int
srtp_init_from_hex(srtp_ctx_t *ctx, sec_serv_t sec_servs,
		   unsigned char *input_key) {
  unsigned char key[SRTP_INIT_MAX_KEY_LEN];
  unsigned char *salt;
  err_status_t stat;
  int len, len_expected;
  
  /* 
   * use manual keying - read key and salt from hexadecimal input
   * string
   */

  len_expected = 30 * 2;  /* 16 + 14 octets of key + salt expected */

  /* read hex string to key/salt buffer */
  len = hex_string_to_octet_string(key, input_key, len_expected);

  /* check that hex string is the right length */
  if (len < len_expected) {
    fprintf(stderr, 
	    "error: too few digits in key/salt "
	    "(should be %d hexadecimal digits, found %d)\n",
	    len_expected, len);
    exit(1);    
  } 
  if (strlen(input_key) > len_expected) {
    fprintf(stderr, 
	    "error: too many digits in key/salt "
	    "(should be %d hexadecimal digits, found %ld)\n",
	    len_expected, strlen(input_key));
    exit(1);    
  }
    
  /* set the salt pointer to the correct location in the key/salt string */
  salt = key + 16;

  printf("set master key/salt to %s/", octet_string_hex_string(key, 32));
  printf("%s\n", octet_string_hex_string(salt, 28));
   
  stat = srtp_init_aes_128_prf(ctx, key, salt);
  if (stat)
    fprintf(stderr, "error: srtp initialization failed with code %d\n", stat);

  return 0;
}

int
srtp_sender_init(rtp_sender_t *sender, struct sockaddr_in name, 
		 sec_serv_t sec_servs, unsigned char *input_key) {
  cipher_type_t *ctype;
  auth_type_t *atype;
  int cipher_key_len = 0;
  int auth_key_len = 0;
  int auth_tag_len = 0;
  err_status_t stat;  

  /*
   * cipher and auth func are hardwired in this function for
   * simplicity's sake
   */
  
  /* choose cipher based on security service flag */
  if (sec_servs) {
    ctype = &rijndael_icm;
    cipher_key_len = 16;
  } else {
    ctype = &null_cipher;
  }
  
  /* choose auth_func based on security service flag */
  if (sec_servs & sec_serv_auth) {
    atype = &tmmhv2;
    auth_key_len = 94;
    auth_tag_len = 4;
  } else {
    atype = &null_auth;
  }
  
  /* allocate srtp context */
  stat = srtp_alloc(&sender->srtp_ctx,
		    ctype,
		    cipher_key_len,
		    atype,
		    auth_key_len,
		    auth_tag_len,
		    sec_servs);
  if (stat) {
    fprintf(stderr, "error: srtp alloc failed with code %d\n", stat);
    exit(1);
  }

  if (sec_servs) {
    stat = srtp_init_from_hex(sender->srtp_ctx, sec_servs, input_key);
    if (stat) {
      fprintf(stderr, "error: srtp init failed with code %d\n", stat);
      exit(1);
    }
  }
    
  return 0;

}


int
srtp_receiver_init(rtp_receiver_t *rcvr, struct sockaddr_in name, 
		   sec_serv_t sec_servs, unsigned char *input_key) {
  err_status_t stat;
  cipher_type_t *ctype = &null_cipher;
  auth_type_t *atype;
  int cipher_key_len = 0;
  int auth_key_len = 0;
  int auth_tag_len = 0;

  /* choose cipher based on security service flag */
  if (sec_servs) {
    ctype = &rijndael_icm;
    cipher_key_len = 16;
  } else {
    ctype = &null_cipher;
  }
  
  /* choose auth_func based on security service flag */
  if (sec_servs & sec_serv_auth) {
    atype = &tmmhv2;
    auth_key_len = 94;
    auth_tag_len = 4;
  } else {
    atype = &null_auth;
  }
  
  /* allocate srtp context */
  stat = srtp_alloc(&rcvr->srtp_ctx,
		    ctype,
		    cipher_key_len,
		    atype,
		    auth_key_len,
		    auth_tag_len,
		    sec_servs);
  if (stat) {
    fprintf(stderr, "error: srtp alloc failed with code %d\n", stat);
    exit(1);
  }

  if (sec_servs) {
    stat = srtp_init_from_hex(rcvr->srtp_ctx, sec_servs, input_key);
    if (stat) {
      fprintf(stderr, "error: srtp init failed with code %d\n", stat);
      exit(1);
    }
  }

  return 0;
}
